package com.galaxis.manatee.task;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.PageRequest;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/1 11:37 上午
 */
@Slf4j
@Service
public class ChuanyunProjectScheduledTask {

    private final Integer pageSize=20; final ObjectMapper objectMapper=new ObjectMapper();
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunSaleContractDao chuanyunSaleContractDao;
    private final ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao;
    private final ChuanyunPurCostDao chuanyunPurCostDao;
    private final ChuanyunProjectCashFlowDao chuanyunProjectCashFlowDao;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunReimbursementDao chuanyunReimbursementDao;
    private final ChuanyunLaborCostDao chuanyunLaborCostDao;
    private final ChuanyunPersonDaysDao chuanyunPersonDaysDao;
    private final ChuanyunProjectDao chuanyunProjectDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunWorkHourDao chuanyunWorkHourDao;
    private final ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao;
    private final ChuanyunPayrollDao chuanyunPayrollDao;
    private final ChuanyunSelfCostDao chuanyunSelfCostDao;
    private final ChuanyunMemberCostDao chuanyunMemberCostDao;
    private final ChuanyunProjectCostDao chuanyunProjectCostDao;

    public ChuanyunProjectScheduledTask(ChuanYunManager chuanYunManager, ChuanyunSaleContractDao chuanyunSaleContractDao, ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao, ChuanyunPurCostDao chuanyunPurCostDao, ChuanyunProjectCashFlowDao chuanyunProjectCashFlowDao, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunReimbursementDao chuanyunReimbursementDao, ChuanyunLaborCostDao chuanyunLaborCostDao, ChuanyunPersonDaysDao chuanyunPersonDaysDao, ChuanyunProjectDao chuanyunProjectDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunWorkHourDao chuanyunWorkHourDao, ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao, ChuanyunPayrollDao chuanyunPayrollDao, ChuanyunSelfCostDao chuanyunSelfCostDao, ChuanyunMemberCostDao chuanyunMemberCostDao, ChuanyunProjectCostDao chuanyunProjectCostDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunSaleContractDao = chuanyunSaleContractDao;
        this.chuanyunPaymentReceiveDao = chuanyunPaymentReceiveDao;
        this.chuanyunPurCostDao = chuanyunPurCostDao;
        this.chuanyunProjectCashFlowDao = chuanyunProjectCashFlowDao;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunReimbursementDao = chuanyunReimbursementDao;
        this.chuanyunLaborCostDao = chuanyunLaborCostDao;
        this.chuanyunPersonDaysDao = chuanyunPersonDaysDao;
        this.chuanyunProjectDao = chuanyunProjectDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunWorkHourDao = chuanyunWorkHourDao;
        this.chuanyunSelfWorkHourDao = chuanyunSelfWorkHourDao;
        this.chuanyunPayrollDao = chuanyunPayrollDao;
        this.chuanyunSelfCostDao = chuanyunSelfCostDao;
        this.chuanyunMemberCostDao = chuanyunMemberCostDao;
        this.chuanyunProjectCostDao = chuanyunProjectCostDao;
    }

    /**
     * 更新销售合同数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSaleContractDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSaleContractDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSaleContractDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同信息保存保存"+totalCount);
    }

    /**
     * 同步项目收付款
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getCashFlow() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectCashFlowDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunProjectCashFlowDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunProjectCashFlowDO -> {
                    var tmp=chuanyunProjectCashFlowDao.findById(chuanyunProjectCashFlowDO.getObjectId());
                    tmp.ifPresent(chuanyunProjectCashFlowDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunProjectCashFlowDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目收付款保存"+totalCount);
    }

    /**
     * 同步项目物料成本
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getPurCost() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunPurCostDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunPurCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunPurCostDO -> {
                    var tmp=chuanyunPurCostDao.findById(chuanyunPurCostDO.getObjectId());
                    tmp.ifPresent(chuanyunPurCostDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunPurCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
            }
        log.info("项目物料成本保存"+totalCount);
    }

    /**
     * 项目核算登记
     */
//    @Scheduled(fixedDelay = 3600000L)
    private void saveAdjustAccount() {
        var objectMapper=new ObjectMapper();
        var flag = true;
        var pageSize = 20;
        var page = 0;
        while (flag) {
            var pageable= PageRequest.of(page,pageSize);
            var chuanyunGroupProjectDOS = chuanyunProjectDao.findAllByIsCreate(null,pageable);
            if (page<=chuanyunGroupProjectDOS.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunProjectDO> list = chuanyunGroupProjectDOS.getContent();
            list.forEach(chuanyunProjectDO -> {
                ChuanyunAdjustAccountDO chuanyunAdjustAccountDO = new ChuanyunAdjustAccountDO();
                //项目号
                chuanyunAdjustAccountDO.setProjectCode(chuanyunProjectDO.getProjectCode());
                //核算实施费用
                BigDecimal money = new BigDecimal(0);
                List<ChuanyunReimbursementDO> list1 = chuanyunReimbursementDao.findAllByGroupProjectId(chuanyunProjectDO.getObjectId());
                if (list1 != null) {
                   for (ChuanyunReimbursementDO chuanyunReimbursementDO : list1) {
                       money = money.add(chuanyunReimbursementDO.getAmount());
                   }
                }
                chuanyunAdjustAccountDO.setImplementationCosts(money);

                //核算人天
                chuanyunAdjustAccountDO.setPersonDays(personDays(chuanyunProjectDO.getObjectId()));
                //新增
                if (!chuanyunAdjustAccountDO.getPersonDays().toString().equals("0")){
                    try {
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000002_2,"+chuanyunProjectDO.getProjectCode());
                        var filter= Filter.instance(0,1,true,"And",macthers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunAdjustAccountDO.SCHEMA_CODE,filter);
                        var adjustString=objectMapper.writeValueAsString(chuanyunAdjustAccountDO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<ChuanyunAdjustAccountDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunAdjustAccountDO chuanyunAdjustAccountDO1 = result.get(0);
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunAdjustAccountDO.SCHEMA_CODE,chuanyunAdjustAccountDO1.getObjectId(),adjustString);
                        }else  {
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunAdjustAccountDO.SCHEMA_CODE,adjustString,true);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
                            log.info("新增项目核算登记成功");
                        }else{
                            log.warn("新增项目核算登记失败");
                        }
                    } catch (JsonProcessingException | BigSizeException e) {
                        e.printStackTrace();
                    }
                }
            });
        }
    }

    /**
     * 更新销售合同回款数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractPaymentReceiveFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunPaymentReceiveDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunPaymentReceiveDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunPaymentReceiveDO -> {
                    var tmp=chuanyunPaymentReceiveDao.findById(chuanyunPaymentReceiveDO.getObjectId());
                    tmp.ifPresent(chuanyunPaymentReceiveDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunPaymentReceiveDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同回款信息保存保存"+totalCount);
    }

    /**
     * 计算人天
     * @param projectId
     * @return
     */
    private BigDecimal personDays(String projectId){
        BigDecimal standard = new BigDecimal(8);
        BigDecimal personDays = new BigDecimal(0);
        //项目每人每天工时
       List<ChuanyunPersonDaysDO> selfWorkHourList = chuanyunPersonDaysDao.getSelfWorkHour(projectId);
       for (ChuanyunPersonDaysDO chuanyunPersonDaysDO : selfWorkHourList) {
           //项目总工时
           ChuanyunPersonDaysDO totalWorkHour = chuanyunPersonDaysDao.getTotalWorkHour(projectId,chuanyunPersonDaysDO.getUserId());
           if (totalWorkHour != null){
               BigDecimal totalDays = (chuanyunPersonDaysDO.getHoursWorked().divide(totalWorkHour.getHoursWorked(), 2, BigDecimal.ROUND_HALF_UP)).multiply(standard);
               personDays= personDays.add(totalDays);
           }
       }
       return personDays;
    }

    /**
     * 人力成本
     */
    @Scheduled(fixedDelay = 3600000L)
    private void laborcost() {
        var flag = true;
        var pageSize = 20;
        var page = 0;
        while(flag) {
            var pageable= PageRequest.of(page,pageSize);
            var chuanyunUserCompanyDOS = chuanyunUserCompanyDao.findAllByStatus(1,pageable);
            if (page<=chuanyunUserCompanyDOS.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunUserCompanyDO> userCompanyDOList = chuanyunUserCompanyDOS.getContent();
            userCompanyDOList.forEach(chuanyunUserCompanyDO -> {
                //工时标准化
                standardHour(chuanyunUserCompanyDO);
                //单项目单人单月成本
                selfCost(chuanyunUserCompanyDO);
            });
            //每人月度汇总
            List<ChuanyunMemberCostDO> list = chuanyunMemberCostDao.getMemberCost();
            list.forEach(chuanyunMemberCostDO -> {
                ChuanyunMemberCostDO memberCostDO = new ChuanyunMemberCostDO();
                memberCostDO.setCost(chuanyunMemberCostDO.getCost());
                memberCostDO.setProjectId(chuanyunMemberCostDO.getProjectId());
                memberCostDO.setUserId(chuanyunMemberCostDO.getUserId());
                memberCostDO.setTotalHour(chuanyunMemberCostDO.getTotalHour());
                chuanyunMemberCostDao.save(memberCostDO);
            });
            //项目汇总
           List<ChuanyunProjectCostDO> list1 = chuanyunProjectCostDao.getProjectCost();
           list1.forEach(chuanyunProjectCostDO -> {
               ChuanyunProjectCostDO projectCostDO = new ChuanyunProjectCostDO();
               projectCostDO.setCost(chuanyunProjectCostDO.getCost());
               projectCostDO.setProjectId(chuanyunProjectCostDO.getProjectId());
               projectCostDO.setTotalHour(chuanyunProjectCostDO.getTotalHour());
               chuanyunProjectCostDao.save(projectCostDO);
           });
        }
    }

    private void standardHour(ChuanyunUserCompanyDO chuanyunUserCompanyDO) {
        BigDecimal standard = new BigDecimal(8);
        //每人每个项目每天工时
        List<ChuanyunWorkHourDO> selfList = chuanyunWorkHourDao.getSelfWorkHour(chuanyunUserCompanyDO.getUserId());
        selfList.forEach(self->{
            ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO = new ChuanyunSelfWorkHourDO();
            chuanyunSelfWorkHourDO.setUserId(chuanyunUserCompanyDO.getUserId());
            chuanyunSelfWorkHourDO.setProjectId(self.getProjectId());
            chuanyunSelfWorkHourDO.setDayLogDate(self.getDayLogDate());
            //每人每天工时
            ChuanyunWorkHourDO chuanyunWorkHourDO = chuanyunWorkHourDao.getDayWorkHour(chuanyunUserCompanyDO.getUserId(),self.getDayLogDate());
            //工时标准化
            if (chuanyunWorkHourDO != null) {
                if (chuanyunWorkHourDO.getHoursWorked().intValue()<8) {
                    chuanyunSelfWorkHourDO.setStandardWorkHour(self.getHoursWorked());
                }else {
                    BigDecimal personDays = (self.getHoursWorked().divide(chuanyunWorkHourDO.getHoursWorked(), 2, BigDecimal.ROUND_HALF_UP)).multiply(standard);
                    chuanyunSelfWorkHourDO.setStandardWorkHour(personDays);
                }
                ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO1 = chuanyunSelfWorkHourDao.findByUserIdAndAndProjectIdAndAndDayLogDate(chuanyunUserCompanyDO.getUserId(),self.getProjectId(),self.getDayLogDate());
                if (chuanyunSelfWorkHourDO1!=null) {
                    chuanyunSelfWorkHourDao.delete(chuanyunSelfWorkHourDO1);
                }
                chuanyunSelfWorkHourDao.save(chuanyunSelfWorkHourDO);
                log.info("工时标准化新增成功");
            }
        });
    }

    private void selfCost(ChuanyunUserCompanyDO chuanyunUserCompanyDO) {
        //每月每项目工时
        List<ChuanyunSelfWorkHourDO> list = chuanyunSelfWorkHourDao.getMonthProjectHour(chuanyunUserCompanyDO.getUserId());
        list.forEach(projectMonth->{
            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            ChuanyunSelfWorkHourDO monthHour = chuanyunSelfWorkHourDao.getMonthHour(projectMonth.getUserId(),df.format(projectMonth.getDayLogDate()));
            ChuanyunPayrollDO chuanyunPayrollDO = chuanyunPayrollDao.getPayroll(projectMonth.getUserId(),df.format(projectMonth.getDayLogDate()));
            ChuanyunSelfCostDO chuanyunSelfCostDO = new ChuanyunSelfCostDO();
            if (chuanyunPayrollDO != null) {
                BigDecimal cost = (projectMonth.getStandardWorkHour().divide(monthHour.getStandardWorkHour(), 2, BigDecimal.ROUND_HALF_UP)).multiply(chuanyunPayrollDO.getWagesPayable());
                chuanyunSelfCostDO.setCost(cost);
                chuanyunSelfCostDO.setProjectId(projectMonth.getProjectId());
                chuanyunSelfCostDO.setUserId(projectMonth.getUserId());
                chuanyunSelfCostDO.setDayLogDate(projectMonth.getDayLogDate());
                chuanyunSelfCostDO.setTotalHour(monthHour.getStandardWorkHour());
                chuanyunSelfCostDao.save(chuanyunSelfCostDO);
                log.info("人力成本计算");
            }
        });
    }
}
